#include <iostream>
#include <cstdlib>  // c++ exit
#include <exception>
#include <typeinfo>
#include "MyWorld.h"
#include "Ball.h"
#include "CVector.h"
#include "Container.h"

using namespace std;

int main(int argvc, char * argv[]) {
	double deltaTime;
	double endTime;
	double samplingTime;
	double gravity;
	double friction;
	try{
		if (argvc == 4) {
			deltaTime = /*0.001; */atof(argv[1]);    // [s]
			endTime = /*10.0;*/atof(argv[2]);      // [s]
			samplingTime = /*0.1; */atof(argv[3]); // [s]
			gravity = friction = 0.0;
		}
		else if (argvc == 5) {
			deltaTime = /*0.001; */atof(argv[1]);    // [s]
			endTime = /*10.0;*/atof(argv[2]);      // [s]
			samplingTime = /*0.1; */atof(argv[3]); // [s]
			gravity = atof(argv[4]);
			friction = 0.0;
		}
		else if (argvc == 6) {
			deltaTime = /*0.001; */atof(argv[1]);    // [s]
			endTime = /*10.0;*/atof(argv[2]);      // [s]
			samplingTime = /*0.1; */atof(argv[3]); // [s]
			gravity = atof(argv[4]);
			friction = atof(argv[5]);
		}
		else {
			cout << "ingresa:\n deltaTime:\t";
			cin >> deltaTime;
			cout << "\n endTime:\t";
			cin >> endTime;
			cout << "\n samplingTime:\t";
			cin >> samplingTime;
			gravity = friction = 0.0;
		}
	}
	catch (...) {
		cout << "usage: " << "< deltaTime >" << "< endTime >" << "< samplingTime >" << endl;
		exit(-1);
	}

    MyWorld world(cout);
      
    double mass = 1.0;      // 1 [kg] 
    double radius = 0.1;    // 10 [cm] 
    CVector position(-2,-2);  // [m] 
    CVector speed(0,0);     //  [m/s]
    Ball b0(mass, radius, position, speed);
	b0.setGravity(gravity);
	b0.setFriction(friction);
    position.set(2,2);
    speed.set(0,0);
    Ball b1(mass, radius, position, speed);
	b1.setGravity(gravity);
	b1.setFriction(friction);
	position.set(2, -2);
	Ball b2(mass, radius, position, speed);
	b2.setGravity(gravity);
	b2.setFriction(friction);
    Container c(CVector(-4,4),CVector(4,-4));
	Spring s1(4, 1);
	s1.attachEnd(&b0);
	s1.attachEnd(&b1);
	Spring s2(3, 1);
	s2.attachEnd(&b0);
	s2.attachEnd(&b2);
	Spring s3(3, 1);
	s3.attachEnd(&b2);
	s3.attachEnd(&b1);
    world.addElement(&b0);
    world.addElement(&b1);
	world.addElement(&b2);
    world.addElement(&c);
	world.addElement(&s1);
	world.addElement(&s2);
	world.addElement(&s3);
    world.simulate(deltaTime, endTime, samplingTime); // delta time[s], total simulation time [s].
}
