#ifndef BALL_H
#define BALL_H

class PhysicsElement; // incomplete class declaration to deal with circular class definition

#include <string>
#include <vector>
#include "CVector.h"
#include "InertialElement.h"
#include "MyWorld.h"
#include "Container.h"
#include "Wall.h"
#include "Spring.h"

using namespace std;

class Ball: public InertialElement {
private:
   static int id;  // Ball identification number
   const double mass;
   const double radius;
   CVector pos_t;     // current position at time t
   CVector pos_tPlusDelta;  // next position in delta time in future
   CVector accel_t;
   CVector speed_t;   // speed at time t
   CVector speed_tPlusDelta;   // speed in delta time in future
   CVector accel_tPlusDelta;
   vector<const PhysicsElement*> collidingWithMe;
   vector<Wall> collidingWall;
   vector<Spring*> attachedSprings;
   Ball();
public:
   Ball(double mass, double radius, CVector position, CVector speed);
   double getRadius() const;
   double getMass() const;
   CVector getSpeed() const;
   CVector getPosition() const;
   void setSpring(Spring *s);
   //CVector getNetForce() const; caso para choque con muros aun no acelera, se plantea definirla en clase spring
   friend ostream & operator<< (ostream &, const Ball &);
   virtual string getDescription() const;
   virtual string getState() const;
   virtual const PhysicsElement * collideWith(const Ball *b) const; 
   virtual void computeNextState(double delta_t, MyWorld * world); 
   virtual void updateState();
};
#endif